﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Microsoft.ResourceManagement.Client.Faults;

namespace Microsoft.ResourceManagement.Client {
    public class ClientHelper
    {
        // Create a logger for use in this class.
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(
                System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        private ClientHelper() {
        }

        public static void HandleFault(Message message) {
            //log.Debug("[HandleFault]\r\n" + message.ToString());
            MessageFault fault = MessageFault.CreateFault(message, Int32.MaxValue);
            FaultException exc = FaultException.CreateFault(fault,
                typeof(PermissionDeniedFault),
                typeof(AuthenticationRequiredFault),
                typeof(AuthorizationRequiredFault),
                typeof(EndpointUnavailable),
                typeof(FragmentDialectNotSupported),
                typeof(InvalidRepresentationFault),
                typeof(UnwillingToPerform),
                typeof(CannotProcessFilter),
                typeof(FilterDialectRequestedUnavailable),
                typeof(UnsupportedExpiration),
                typeof(AnonymousInteractionRequiredFault)
            );
//            log.DebugFormat(
//@"[HandleFault]
//Action =               {0}
//Code.Name =            {1}
//Code.SubCode.Name =    {2}
//Message =              {3}
//Reason =               {4}
//Source =               {5}",
//                exc.Action,
//                exc.Code.Name,
//                exc.Code.SubCode.Name,
//                exc.Message,
//                exc.Reason,
//                exc.Source);
            throw exc;
        }

        public static void AddRmHeaders(WsTransfer.TransferRequest transferRequest, Message message) {
            if (transferRequest == null)
                return;

            if (transferRequest.ResourceLocaleProperty != null && String.IsNullOrEmpty(transferRequest.ResourceLocaleProperty.Value) == false) {
                if (message.Headers.FindHeader(Constants.Rm.ResourceLocaleProperty, Constants.Rm.Namespace) < 0) {
                    MessageHeader newHeader = MessageHeader.CreateHeader(Constants.Rm.ResourceLocaleProperty, Constants.Rm.Namespace, transferRequest.ResourceLocaleProperty.Value);
                    message.Headers.Add(newHeader);
                }
            }

            if (transferRequest.ResourceReferenceProperty != null && String.IsNullOrEmpty(transferRequest.ResourceReferenceProperty.Value) == false) {
                if (message.Headers.FindHeader(Constants.Rm.ResourceReferenceProperty, Constants.Rm.Namespace) < 0) {
                    MessageHeader newHeader = MessageHeader.CreateHeader(Constants.Rm.ResourceReferenceProperty, Constants.Rm.Namespace, transferRequest.ResourceReferenceProperty.Value);
                    message.Headers.Add(newHeader);
                }
            }

            if (transferRequest.ResourceTimeProperty != null && String.IsNullOrEmpty(transferRequest.ResourceTimeProperty.Value) == false) {
                if (message.Headers.FindHeader(Constants.Rm.ResourceTimeProperty, Constants.Rm.Namespace) < 0) {
                    MessageHeader newHeader = MessageHeader.CreateHeader(Constants.Rm.ResourceTimeProperty, Constants.Rm.Namespace, transferRequest.ResourceTimeProperty.Value);
                    message.Headers.Add(newHeader);
                }
            }
        }

        public static void AddImdaHeaders(WsTransfer.ImdaRequest imdaRequest, Message message) {
            if (imdaRequest == null)
                return;
            if (message == null) {
                throw new ArgumentNullException("message");
            }
            if (message.Headers.FindHeader(Constants.Imda.ExtensionHeaderName, Constants.Imda.Namespace) < 0) {
                MessageHeader newHeader = MessageHeader.CreateHeader(Constants.Imda.ExtensionHeaderName, Constants.Imda.Namespace, String.Empty, true);
                message.Headers.Add(newHeader);
            }
        }
    }
}
